#include "pcy_sk.hpp"
#include "pcl_sk.hpp"
#include "pce_sk.hpp"

ABXError::ABXError(ABXLex *LexObject, ABXYacc *YaccObject)
{
	lexObject = LexObject;
	yaccObject = YaccObject;
	yyErrCnt = 0;
	yyErrSrc = new char[FILENAMELEN];
//	FILE *yyErrFile;
	yyErrLineNo = -1;
	char *yyErrText = new char[TEXTLEN];
}

ABXError::~ABXError()
{
	delete[] yyErrSrc;
	delete[] yyErrText;
}

/* This function takes a token value (integer) as input and returns 
an actual token value expressed in character string. A user defined 
token list should be provided in order for this function to work. 
*/
char * ABXError::yyGetToken(int tokenvalue)
{
	/* yydisplay is defined by user according to the tokens used 
	   for your grammar */
	return yydisplay(tokenvalue);
}

/* This function returns current line number that the lexical scanner is 
currently processing.
*/
int ABXError::yyGetLineNumber(void)
{
	yyErrLineNo = lexObject->get_yyLineNo();
	return yyErrLineNo;
}

/* This function returns current error number that the lexical scanner is 
currently processing.
*/
int ABXError::yyGetErrorCount(void)
{
	yyErrCnt = yaccObject->get_yyErrorCount();
	return yyEreCnt;
}

/* This function will take input parameter to set error file name.
*/
void ABXError::yyGetErrFileName(char *s)
{
	strcpy(yyErrSrc, s);
}

/* This function will use input parameter to set yyErrText(error message).
*/
void ABXError::yySetErrText(char *s)
{
	strcpy(yyErrText, s);
}

/* This function returns the beginning position of current token on the 
current line the lexical scanner is processing.
*/
int ABXError::yyGetCharNumber(void)
{
	return lexObject->yyBufferPtrC;
}

/* This function takes an integer array of internal values for expected
tokens as input and returns the exact number of those expected tokens. 
*/
int ABXError::yyGetExpectedTokens(int *token_list)
{
	return sizeof(token_list)/sizeof(int*);
}

/* This function takes a syntax error message as input and displays
it according to the current token information. The current token
information including line number, char position number, ..., etc, 
is provided by the lexer. 
*/
void ABXError::yyErrPrefix(char *msg, char *YYTEXT)
{
	int punct = 0;

	/* set yyerrfile, yyerrcnt, yyerrsrc and yylineno to its 
	corresponding value*/
	yyErrFile = yyerrfile;
	yyErrCnt = yaccObject->get_yyErrorCount();
	strcpy(yyErrSrc, yyerrsrc);
	yyLineNo = lexObject->get_yyLineNo();

	fprintf(yyErrFile, "error %d]", yyErrCnt+1);

	if (yyErrSrc[0]!='\0')   /* any input file name? */
	{
		fprintf(yyErrFile, "file '%s'", yyErrSrc);
		punct = 1;
	}

	if (yyLineNo>=0)    /* valid line number? */
	{
		if (punct)
			fprintf(yyErrFile, ",");
		punct = 1;
	}

	if (YYTEXT!=NULL && *YYTEXT!='\0')    /* real token? */
	{
		if (punct)
			fprintf(yyErrFile, " ");
		fprintf(yyErrFile, "near \"%s\"", YYTEXT);
		punct = 1;
	}

	if (punct)
		fprintf(yyErrFile, ": ");

	fprintf(yyErrFile, "%s\n", msg);

}

/* This function simply calls yyErrPrefix to finish error 
reporting by reformatting the error message display. The two 
input strings can represent two different error messages to 
be displayed. 
*/
void ABXError::yyError(char *s, char *t, char *YYTEXT)
{
	static const char expecting[] = "expecting: ";
	static int list = 0;
	static int column = 0;

	yyErrFile = yyerrfile;

	if (s!=NULL)
	{
		if (column!=0)
			fputc('\n', yyErrFile);
		
		yyErrPrefix(lexObject, yaccObject, s, YYTEXT);

		if (t==NULL)
			column = 0;
		else
			column = fprintf(yyerrfile, "actual" %s", t);
			list = 0;
	}
	else if (t!=NULL)
	{
		if (list==0)
		{
			if (column+strlen(t)+sizeof(expecting)+1<WIDTH-2)
				column += fprintf(yyErrFile, " %s%s", expecting, t);
			else
				column = fprintf(yyErrFile, "\n%s%s", expecting, t) -1;
		}
		else
		{
			if (column+strlen(t)<WIDTH-2)
				column += fprintf(yyErrFile, ", %s", t);
			else
				column = fprintf(yyErrFile, ",\n %s", t) - 1;
		}
		++list;
	}
	else
	{
		fprintf(yyErrFile, "\n");
		column = list = 0;
	}

}

/* This function makes file indicator forward one unit to skip
current token as if the current token being processed by lexer
does not exist in the input stream. 
*/
void ABXError::yySkipToken(void)
{
	lexObject->yyLex();    /* skip current token */
}

/* This function simply skips the current symbol in the sentential 
form if it is a terminal and keeps the current token unchanged.
*/
void ABXError::yySkipSymbol(void)
{
}

/* This function takes a replacing token as input and replaces the 
token that will be processed by the lexer. 
*/
void ABXError::yyReplaceToken(char *token)
{
	int i = 0;

    lexObject->yyLex();
	while (token[i]!='\0')
	{
		lexObject->unput(token[i]);
	    i++;
	}
}

/* This function takes a matching token as input parameter. 
It will return 1 if the current token matches the actual 
input token value, otherwise returns 0 instead. 
*/
int ABXError::yyMatchToken(char *token)
{
	if (strcmp(lexObject->get_yyText(), token)==0)
		return 1;
	else
		return 0;
}

/* This function takes a token to be inserted as input parameter. 
Its purpose is to insert a token in front of the one that will
be processed by the lexer.
*/
void ABXError::yyInsertToken(char *token)
{
	int i = 0;

	while (token[i]!='\0')
	{
		lexObject->unput(token[i]);
	    i++;
	}

}
